@@ -1,28 +1,33 @@
 # Copyright (C) 2005-2007 Jelmer Vernooij <jelmer@samba.org>
  
 # This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
+# it under the terms of the GNU Lesser General Public License as published by
+# the Free Software Foundation; either version 2.1 of the License, or
 # (at your option) any later version.
 
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
+# GNU Lesser General Public License for more details.
 
-# You should have received a copy of the GNU General Public License
+# You should have received a copy of the GNU Lesser General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 """Subversion ra library tests."""
 
 from unittest import TestCase
-from subvertpy import wc
+
+from subvertpy import (
+    wc,
+    )
 
 class VersionTest(TestCase):
+
     def test_version_length(self):
         self.assertEquals(4, len(wc.version()))
 
 class WorkingCopyTests(TestCase):
+
     def test_get_adm_dir(self):
         self.assertEquals(".svn", wc.get_adm_dir())
 
