@@ -3,16 +3,16 @@
  * -*- coding: utf-8 -*-
  *
  * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * it under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
+ * You should have received a copy of the GNU Lesser General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
@@ -20,6 +20,7 @@
 #include <apr_general.h>
 #include <svn_wc.h>
 #include <svn_path.h>
+#include <svn_props.h>
 #include <structmember.h>
 #include <stdbool.h>
 
@@ -37,6 +38,7 @@ static svn_error_t *py_ra_report_set_path(void *baton, const char *path, svn_rev
 	PyGILState_STATE state = PyGILState_Ensure();
 	if (lock_token == NULL) {
 		py_lock_token = Py_None;
+		Py_INCREF(py_lock_token);
 	} else {
 		py_lock_token = PyString_FromString(lock_token);
 	}
@@ -62,6 +64,7 @@ static svn_error_t *py_ra_report_link_path(void *report_baton, const char *path,
 	PyGILState_STATE state = PyGILState_Ensure();
 	if (lock_token == NULL) {
 		py_lock_token = Py_None;
+		Py_INCREF(py_lock_token);
 	} else { 
 		py_lock_token = PyString_FromString(lock_token);
 	}
@@ -159,6 +162,8 @@ static PyMemberDef entry_members[] = {
 		"Copyfrom location" },
 	{ "copyfrom_rev", T_LONG, offsetof(EntryObject, entry.copyfrom_rev), READONLY, 
 		"Copyfrom revision" },
+	{ "uuid", T_STRING, offsetof(EntryObject, entry.uuid), READONLY,
+		"UUID of repository" },
 	{ "url", T_STRING, offsetof(EntryObject, entry.url), READONLY, 
 		"URL in repository" },
 	{ "repos", T_STRING, offsetof(EntryObject, entry.repos), READONLY, 
@@ -244,7 +249,12 @@ PyTypeObject Entry_Type = {
 
 static PyObject *py_entry(const svn_wc_entry_t *entry)
 {
-	EntryObject *ret = PyObject_New(EntryObject, &Entry_Type);
+	EntryObject *ret;
+
+	if (entry == NULL)
+		Py_RETURN_NONE;
+	
+	ret = PyObject_New(EntryObject, &Entry_Type);
 	if (ret == NULL)
 		return NULL;
 
@@ -331,6 +341,7 @@ static PyObject *adm_prop_get(PyObject *self, PyObject *args)
 	RUN_SVN_WITH_POOL(temp_pool, svn_wc_prop_get(&value, name, path, admobj->adm, temp_pool));
 	if (value == NULL || value->data == NULL) {
 		ret = Py_None;
+		Py_INCREF(ret);
 	} else {
 		ret = PyString_FromStringAndSize(value->data, value->len);
 	}
@@ -371,7 +382,7 @@ static PyObject *adm_entries_read(PyObject *self, PyObject *args)
 	const char *key;
 	apr_ssize_t klen;
 	svn_wc_entry_t *entry;
-	PyObject *py_entries;
+	PyObject *py_entries, *obj;
 
 	if (!PyArg_ParseTuple(args, "|b", &show_hidden))
 		return NULL;
@@ -385,7 +396,9 @@ static PyObject *adm_entries_read(PyObject *self, PyObject *args)
 	idx = apr_hash_first(temp_pool, entries);
 	while (idx != NULL) {
 		apr_hash_this(idx, (const void **)&key, &klen, (void **)&entry);
-		PyDict_SetItemString(py_entries, key, py_entry(entry));
+		obj = py_entry(entry);
+		PyDict_SetItemString(py_entries, key, obj);
+		Py_DECREF(obj);
 		idx = apr_hash_next(idx);
 	}
 	apr_pool_destroy(temp_pool);
@@ -430,7 +443,7 @@ static PyObject *adm_entry(PyObject *self, PyObject *args)
 	temp_pool = Pool(NULL);
 	if (temp_pool == NULL)
 		return NULL;
-	RUN_SVN_WITH_POOL(temp_pool, svn_wc_entry(&entry, path, admobj->adm, show_hidden, temp_pool));
+	RUN_SVN_WITH_POOL(temp_pool, svn_wc_entry(&entry, svn_path_canonicalize(path, temp_pool), admobj->adm, show_hidden, temp_pool));
 	apr_pool_destroy(temp_pool);
 
 	return py_entry(entry);
@@ -475,10 +488,12 @@ static PyObject *adm_get_prop_diffs(PyObject *self, PyObject *args)
 	return Py_BuildValue("(NN)", py_propchanges, py_orig_props);
 }
 
-static PyObject *adm_add(PyObject *self, PyObject *args)
+static PyObject *adm_add(PyObject *self, PyObject *args, PyObject *kwargs)
 {
 	char *path, *copyfrom_url=NULL;
 	svn_revnum_t copyfrom_rev=-1; 
+	char *kwnames[] = { "path", "copyfrom_url", "copyfrom_rev", "cancel_func", 
+		                "notify_func", NULL };
 	PyObject *cancel_func=Py_None, *notify_func=Py_None;
 	AdmObject *admobj = (AdmObject *)self;
 	apr_pool_t *temp_pool;
@@ -487,10 +502,11 @@ static PyObject *adm_add(PyObject *self, PyObject *args)
 	if (temp_pool == NULL)
 		return NULL;
 
-	if (!PyArg_ParseTuple(args, "s|zlOO", &path, &copyfrom_url, &copyfrom_rev, &cancel_func, &notify_func))
+	if (!PyArg_ParseTupleAndKeywords(args, kwargs, "s|zlOO", kwnames, &path, &copyfrom_url, &copyfrom_rev, &cancel_func, &notify_func))
 		return NULL;
 
-	RUN_SVN_WITH_POOL(temp_pool, svn_wc_add2(path, admobj->adm, copyfrom_url, 
+	RUN_SVN_WITH_POOL(temp_pool, svn_wc_add2(
+						   svn_path_canonicalize(path, temp_pool), admobj->adm, copyfrom_url, 
 							copyfrom_rev, py_cancel_func, 
 							(void *)cancel_func,
 							py_wc_notify_func, 
@@ -523,23 +539,43 @@ static PyObject *adm_copy(PyObject *self, PyObject *args)
 	Py_RETURN_NONE;
 }
 
-static PyObject *adm_delete(PyObject *self, PyObject *args)
+static PyObject *adm_delete(PyObject *self, PyObject *args, PyObject *kwargs)
 {
 	AdmObject *admobj = (AdmObject *)self;
 	apr_pool_t *temp_pool;
+	char *kwnames[] = { "path", "cancel_func", "notify_func", "keep_local",
+		                NULL };
 	char *path;
 	PyObject *cancel_func=Py_None, *notify_func=Py_None;
+	bool keep_local = false;
 
-	if (!PyArg_ParseTuple(args, "s|OO", &path, &cancel_func, &notify_func))
+	if (!PyArg_ParseTupleAndKeywords(args, kwargs, "s|OOb", kwnames, 
+									 &path, &cancel_func, &notify_func, 
+									 &keep_local))
 		return NULL;
 
 	temp_pool = Pool(NULL);
 	if (temp_pool == NULL)
 		return NULL;
+
+#if SVN_VER_MAJOR >= 1 && SVN_VER_MINOR >= 5
+	RUN_SVN_WITH_POOL(temp_pool, svn_wc_delete3(path, admobj->adm, 
+							py_cancel_func, (void *)cancel_func,
+							py_wc_notify_func, (void *)notify_func, 
+							keep_local,
+							temp_pool));
+#else
+	if (keep_local) {
+		PyErr_SetString(PyExc_NotImplementedError, 
+						"keep_local not supported on Subversion < 1.5");
+		return NULL;
+	}
+
 	RUN_SVN_WITH_POOL(temp_pool, svn_wc_delete2(path, admobj->adm, 
 							py_cancel_func, (void *)cancel_func,
 							py_wc_notify_func, (void *)notify_func, 
 							temp_pool));
+#endif
 	apr_pool_destroy(temp_pool);
 
 	Py_RETURN_NONE;
@@ -700,6 +736,27 @@ static PyObject *adm_repr(PyObject *self)
 							   svn_wc_adm_access_path(admobj->adm));
 }
 
+static PyObject *adm_remove_lock(PyObject *self, PyObject *args)
+{
+	char *path;
+	AdmObject *admobj = (AdmObject *)self;
+	apr_pool_t *temp_pool;
+
+	if (!PyArg_ParseTuple(args, "s", &path))
+		return NULL;
+
+	temp_pool = Pool(NULL);
+	if (temp_pool == NULL)
+		return NULL;
+
+
+	RUN_SVN_WITH_POOL(temp_pool, svn_wc_remove_lock(path, admobj->adm, temp_pool))
+
+	apr_pool_destroy(temp_pool);
+
+	Py_RETURN_NONE;
+}
+
 static PyMethodDef adm_methods[] = { 
 	{ "prop_set", adm_prop_set, METH_VARARGS, "S.prop_set(name, value, path, skip_checks=False)" },
 	{ "access_path", (PyCFunction)adm_access_path, METH_NOARGS, 
@@ -714,9 +771,9 @@ static PyMethodDef adm_methods[] = {
 		"S.locked() -> bool" },
 	{ "get_prop_diffs", adm_get_prop_diffs, METH_VARARGS, 
 		"S.get_prop_diffs(path) -> (propchanges, originalprops)" },
-	{ "add", adm_add, METH_VARARGS, "S.add(path, copyfrom_url=None, copyfrom_rev=-1, cancel_func=None, notify_func=None" },
+	{ "add", (PyCFunction)adm_add, METH_VARARGS|METH_KEYWORDS, "S.add(path, copyfrom_url=None, copyfrom_rev=-1, cancel_func=None, notify_func=None)" },
 	{ "copy", adm_copy, METH_VARARGS, "S.copy(src_path, dest_path, cancel_func=None, notify_func=None" },
-	{ "delete", adm_delete, METH_VARARGS, "S.delete(path, cancel_func=None, notify_func=None)" },
+	{ "delete", (PyCFunction)adm_delete, METH_VARARGS|METH_KEYWORDS, "S.delete(path, cancel_func=None, notify_func=None, keep_local=False)" },
 	{ "crawl_revisions", (PyCFunction)adm_crawl_revisions, METH_VARARGS|METH_KEYWORDS, 
 		"S.crawl_revisions(path, reporter, restore_files=True, recurse=True, use_commit_times=True, notify_func=None) -> None" },
 	{ "get_update_editor", adm_get_update_editor, METH_VARARGS, NULL },
@@ -725,6 +782,7 @@ static PyMethodDef adm_methods[] = {
 	{ "entry", (PyCFunction)adm_entry, METH_VARARGS, 
 		"s.entry(path, show_hidden=False) -> entry" },
 	{ "process_committed", (PyCFunction)adm_process_committed, METH_VARARGS|METH_KEYWORDS, "S.process_committed(path, recurse, new_revnum, rev_date, rev_author, wcprop_changes=None, remove_lock=False, digest=None)" },
+	{ "remove_lock", (PyCFunction)adm_remove_lock, METH_VARARGS, "S.remove_lock(path)" }, 
 	{ NULL, }
 };
 
@@ -906,7 +964,7 @@ static PyObject *get_pristine_copy_path(PyObject *self, PyObject *args)
 	pool = Pool(NULL);
 	if (pool == NULL)
 		return NULL;
-	RUN_SVN_WITH_POOL(pool, svn_wc_get_pristine_copy_path(path, &pristine_path, pool));
+	RUN_SVN_WITH_POOL(pool, svn_wc_get_pristine_copy_path(svn_path_canonicalize(path, pool), &pristine_path, pool));
 	ret = PyString_FromString(pristine_path);
 	apr_pool_destroy(pool);
 	return ret;
@@ -950,9 +1008,33 @@ static PyObject *check_wc(PyObject *self, PyObject *args)
 	return PyLong_FromLong(wc_format);
 }
 
+static PyObject *cleanup_wc(PyObject *self, PyObject *args, PyObject *kwargs)
+{
+	PyObject *cancel_func = Py_None;
+	char *path;
+	char *diff3_cmd = NULL;
+	char *kwnames[] = { "path", "diff3_cmd", "cancel_func", NULL };
+	apr_pool_t *temp_pool;
+
+	if (!PyArg_ParseTupleAndKeywords(args, kwargs, "s|zO", kwnames, 
+									 &path, &diff3_cmd, &cancel_func)) 
+		return NULL;
+
+	temp_pool = Pool(NULL);
+	if (temp_pool == NULL)
+		return NULL;
+	RUN_SVN_WITH_POOL(temp_pool, 
+				svn_wc_cleanup2(path, diff3_cmd, py_cancel_func, cancel_func,
+								temp_pool));
+	apr_pool_destroy(temp_pool);
+
+	Py_RETURN_NONE;
+}
+
 static PyMethodDef wc_methods[] = {
 	{ "check_wc", check_wc, METH_VARARGS, "check_wc(path) -> bool\n"
 		"Check whether path contains a Subversion working copy" },
+	{ "cleanup", (PyCFunction)cleanup_wc, METH_VARARGS|METH_KEYWORDS, "cleanup(path, diff3_cmd=None, cancel_func=None)\n" },
 	{ "ensure_adm", (PyCFunction)ensure_adm, METH_KEYWORDS|METH_VARARGS, 
 		"ensure_adm(path, uuid, url, repos=None, rev=None)" },
 	{ "get_adm_dir", (PyCFunction)get_adm_dir, METH_NOARGS, 
@@ -1007,6 +1089,23 @@ void initwc(void)
 	PyModule_AddIntConstant(mod, "SCHEDULE_DELETE", 2);
 	PyModule_AddIntConstant(mod, "SCHEDULE_REPLACE", 3);
 
+#if SVN_VER_MAJOR >= 1 && SVN_VER_MINOR >= 5
+	PyModule_AddIntConstant(mod, "CONFLICT_CHOOSE_POSTPONE",
+							svn_wc_conflict_choose_postpone);
+	PyModule_AddIntConstant(mod, "CONFLICT_CHOOSE_BASE",
+							svn_wc_conflict_choose_base);
+	PyModule_AddIntConstant(mod, "CONFLICT_CHOOSE_THEIRS_FULL",
+							svn_wc_conflict_choose_theirs_full);
+	PyModule_AddIntConstant(mod, "CONFLICT_CHOOSE_MINE_FULL",
+							svn_wc_conflict_choose_mine_full);
+	PyModule_AddIntConstant(mod, "CONFLICT_CHOOSE_THEIRS_CONFLICT",
+							svn_wc_conflict_choose_theirs_conflict);
+	PyModule_AddIntConstant(mod, "CONFLICT_CHOOSE_MINE_CONFLICT",
+							svn_wc_conflict_choose_mine_conflict);
+	PyModule_AddIntConstant(mod, "CONFLICT_CHOOSE_MERGED",
+							svn_wc_conflict_choose_merged);
+#endif
+
 	PyModule_AddObject(mod, "WorkingCopy", (PyObject *)&Adm_Type);
 	Py_INCREF(&Adm_Type);
 }
